@@ -114,19 +114,18 @@ class KeycloakOpenIDConnection(ConnectionManager):
         self.client_secret_key = client_secret_key
         self.user_realm_name = user_realm_name
         self.timeout = timeout
+        self.headers = {}
+        self.custom_headers = custom_headers
 
         if self.token is None:
             self.get_token()
 
-        self.headers = (
-            {
+        if self.token is not None:
+            self.headers = {
+                **self.headers,
                 "Authorization": "Bearer " + self.token.get("access_token"),
                 "Content-Type": "application/json",
             }
-            if self.token is not None
-            else {}
-        )
-        self.custom_headers = custom_headers
 
         super().__init__(
             base_url=self.server_url, headers=self.headers, timeout=60, verify=self.verify
@@ -301,6 +300,7 @@ class KeycloakOpenIDConnection(ConnectionManager):
                 verify=self.verify,
                 client_secret_key=self.client_secret_key,
                 timeout=self.timeout,
+                custom_headers=self.custom_headers,
             )
 
         return self._keycloak_openid
@@ -311,10 +311,10 @@ class KeycloakOpenIDConnection(ConnectionManager):
         The admin token is then set in the `token` attribute.
         """
         grant_type = []
-        if self.client_secret_key:
-            grant_type.append("client_credentials")
-        elif self.username and self.password:
+        if self.username and self.password:
             grant_type.append("password")
+        elif self.client_secret_key:
+            grant_type.append("client_credentials")
 
         if grant_type:
             self.token = self.keycloak_openid.token(
